#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>    
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>	
    [Description("Enter the description of your new custom indicator here")]
    public class ImpStdDev : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int length = 20;
        private int dataType = 1;
        // User defined variables (add any user defined variables below)
		private int divisor;
		private SMA mean;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "StdDev"));
            Overlay				= false;
        }
		
		protected override void OnStartUp()
		{
			divisor = dataType==1 ? length : length-1;
			mean = SMA(Input,length);
		}

        protected override void OnBarUpdate()
        {
			if (CurrentBar<length)
				return;
			
			double variance = 0;
            if (divisor>0)
			{
				double sumSqr = 0;
				for (int value1=0;value1<length;value1++)
				{
					double dev = Math.Max(High[value1]-mean[0],mean[0]-Low[value1]);
					sumSqr += dev*dev;
				}
				variance = sumSqr/divisor;
			}
			SDev[0] = variance>0 ? Math.Sqrt(variance) : 0;
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SDev
        {
            get { return Values[0]; }
        }

        [Description("1 - entire population, 2 - sample of population")]
        [GridCategory("Parameters")]
        public int DataType
        {
            get { return dataType; }
            set { dataType = Math.Min(2,Math.Max(1, value)); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ImpStdDev[] cacheImpStdDev = null;

        private static ImpStdDev checkImpStdDev = new ImpStdDev();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ImpStdDev ImpStdDev(int dataType, int length)
        {
            return ImpStdDev(Input, dataType, length);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ImpStdDev ImpStdDev(Data.IDataSeries input, int dataType, int length)
        {
            if (cacheImpStdDev != null)
                for (int idx = 0; idx < cacheImpStdDev.Length; idx++)
                    if (cacheImpStdDev[idx].DataType == dataType && cacheImpStdDev[idx].Length == length && cacheImpStdDev[idx].EqualsInput(input))
                        return cacheImpStdDev[idx];

            lock (checkImpStdDev)
            {
                checkImpStdDev.DataType = dataType;
                dataType = checkImpStdDev.DataType;
                checkImpStdDev.Length = length;
                length = checkImpStdDev.Length;

                if (cacheImpStdDev != null)
                    for (int idx = 0; idx < cacheImpStdDev.Length; idx++)
                        if (cacheImpStdDev[idx].DataType == dataType && cacheImpStdDev[idx].Length == length && cacheImpStdDev[idx].EqualsInput(input))
                            return cacheImpStdDev[idx];

                ImpStdDev indicator = new ImpStdDev();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DataType = dataType;
                indicator.Length = length;
                Indicators.Add(indicator);
                indicator.SetUp();

                ImpStdDev[] tmp = new ImpStdDev[cacheImpStdDev == null ? 1 : cacheImpStdDev.Length + 1];
                if (cacheImpStdDev != null)
                    cacheImpStdDev.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheImpStdDev = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpStdDev ImpStdDev(int dataType, int length)
        {
            return _indicator.ImpStdDev(Input, dataType, length);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpStdDev ImpStdDev(Data.IDataSeries input, int dataType, int length)
        {
            return _indicator.ImpStdDev(input, dataType, length);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpStdDev ImpStdDev(int dataType, int length)
        {
            return _indicator.ImpStdDev(Input, dataType, length);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpStdDev ImpStdDev(Data.IDataSeries input, int dataType, int length)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ImpStdDev(input, dataType, length);
        }
    }
}
#endregion
